Package: RoBMA
Title: Robust Bayesian Meta-Analyses
Version: 4.0.0
Maintainer: František Bartoš <f.bartos96@gmail.com>
Authors@R: c( 
    person("František", "Bartoš",     role = c("aut", "cre"),
    email   = "f.bartos96@gmail.com", comment = c(ORCID = "0000-0002-0018-5573")),
    person("Maximilian", "Maier",     role = "aut",
    email   = "maximilianmaier0401@gmail.com", comment = c(ORCID = "0000-0002-9873-6096")),
    person("Eric-Jan", "Wagenmakers", role = "ths",
    comment = c(ORCID = "0000-0003-1596-1034")),
    person("Joris", "Goosen",         role = "ctb"),
    person("Matthew", "Denwood", role="cph", 
           comment="Original copyright holder of some modified code where indicated."),
    person("Martyn", "Plummer", role="cph", 
           comment="Original copyright holder of some modified code where indicated.")
    )
Description: A framework for Bayesian meta-analysis, including model estimation,
      prior specification, model comparison, prediction, summaries, visualizations,
      and diagnostics. The package fits single and model-averaged meta-analytic,
      meta-regression, multilevel, publication bias adjusted, and generalized linear 
      mixed models The model-averaged meta-analytic models combine competing models 
      based on their predictive performance, weight inference by posterior model probabilities, 
      and test model components using Bayes factors (e.g., effect vs. no effect; 
      Bartoš et al., 2022, <doi:10.1002/jrsm.1594>; Maier, Bartoš & Wagenmakers, 2022,
      <doi:10.1037/met0000405>; Bartoš et al., 2025, <doi:10.1037/met0000737>).
      Users can specify flexible prior distributions for effect sizes, heterogeneity,
      publication bias (including selection models and PET-PEESE), and moderators.
URL: https://fbartos.github.io/RoBMA/
BugReports: https://github.com/FBartos/RoBMA/issues
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
SystemRequirements: JAGS >= 4.3.1 (https://mcmc-jags.sourceforge.io/)
NeedsCompilation: yes
Depends: R (>= 4.0.0)
Imports: BayesTools (>= 0.3.0), bridgesampling, loo, MASS, parallel,
        runjags, rjags, stats, graphics, mvtnorm, scales, Rdpack,
        rlang, coda, ggplot2
Suggests: metafor, posterior, weightr, lme4, fixest, metaBMA, emmeans,
        metadat, testthat, vdiffr, knitr, rmarkdown, covr
LinkingTo: mvtnorm
RdMacros: Rdpack
VignetteBuilder: knitr
Packaged: 2026-05-07 09:33:12 UTC; fbart
Author: František Bartoš [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0018-5573>),
  Maximilian Maier [aut] (ORCID: <https://orcid.org/0000-0002-9873-6096>),
  Eric-Jan Wagenmakers [ths] (ORCID:
    <https://orcid.org/0000-0003-1596-1034>),
  Joris Goosen [ctb],
  Matthew Denwood [cph] (Original copyright holder of some modified code
    where indicated.),
  Martyn Plummer [cph] (Original copyright holder of some modified code
    where indicated.)
Repository: CRAN
Date/Publication: 2026-05-07 13:31:05 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-07 14:15:46 UTC; unix
Archs: RoBMA.so.dSYM
