# Generated by roxygen2: do not edit by hand

S3method(add_loo,brma)
S3method(add_marglik,brma)
S3method(add_waic,brma)
S3method(as.matrix,brma_samples)
S3method(as_draws,brma)
S3method(as_draws,brma_samples)
S3method(as_draws,default)
S3method(as_draws_array,brma)
S3method(as_draws_array,brma_samples)
S3method(as_draws_array,default)
S3method(as_draws_df,brma)
S3method(as_draws_df,brma_samples)
S3method(as_draws_df,default)
S3method(as_draws_list,brma)
S3method(as_draws_list,brma_samples)
S3method(as_draws_list,default)
S3method(as_draws_matrix,brma)
S3method(as_draws_matrix,brma_samples)
S3method(as_draws_matrix,default)
S3method(as_draws_rvars,brma)
S3method(as_draws_rvars,brma_samples)
S3method(as_draws_rvars,default)
S3method(as_zplot,brma)
S3method(bayes_factor,brma)
S3method(bf,brma)
S3method(blup,brma)
S3method(bridge_sampler,brma)
S3method(bridgesampling::bayes_factor,brma)
S3method(bridgesampling::bf,brma)
S3method(bridgesampling::bridge_sampler,brma)
S3method(bridgesampling::logml,brma)
S3method(bridgesampling::post_prob,brma)
S3method(check_loo,brma)
S3method(coef,brma)
S3method(cooks.distance,brma)
S3method(covratio,brma)
S3method(dfbetas,brma)
S3method(dffits,brma)
S3method(fitted,brma)
S3method(funnel,brma)
S3method(galbraith,brma)
S3method(hatvalues,brma)
S3method(hist,zplot_brma)
S3method(influence,brma)
S3method(interpret,brma)
S3method(interpret,default)
S3method(lines,zplot_brma)
S3method(logLik,brma)
S3method(logml,brma)
S3method(loo,brma)
S3method(loo::waic,brma)
S3method(loo_compare,brma)
S3method(loo_compare,loo)
S3method(loo_weights,brma)
S3method(marginal_means,brma)
S3method(metafor::funnel,brma)
S3method(metafor::regplot,brma)
S3method(nobs,brma)
S3method(plot,brma)
S3method(plot,marginal_means.brma)
S3method(plot,only_priors.brma)
S3method(plot,zplot_brma)
S3method(plot_diagnostic,brma)
S3method(plot_diagnostic_autocorrelation,brma)
S3method(plot_diagnostic_density,brma)
S3method(plot_diagnostic_trace,brma)
S3method(plot_pet_peese,brma)
S3method(plot_prior,brma)
S3method(plot_prior,prior)
S3method(plot_weightfunction,brma)
S3method(pooled_effect,brma)
S3method(pooled_heterogeneity,brma)
S3method(post_prob,brma)
S3method(predict,brma)
S3method(print,RoBMA_data)
S3method(print,brma)
S3method(print,brma_samples)
S3method(print,covratio.brma)
S3method(print,dfbetas.brma)
S3method(print,infl.brma)
S3method(print,interpret.brma)
S3method(print,logLik.brma)
S3method(print,marginal_means.brma)
S3method(print,only_priors.brma)
S3method(print,summary.brma)
S3method(print,summary.brma_samples)
S3method(print,summary.marginal_means.brma)
S3method(print,summary.zplot_brma)
S3method(print,summary_heterogeneity.brma)
S3method(print,summary_models.RoBMA)
S3method(print,vif.brma)
S3method(print,zplot_brma)
S3method(print_prior,brma)
S3method(print_prior,prior)
S3method(qqnorm,brma)
S3method(radial,brma)
S3method(ranef,brma)
S3method(regplot,brma)
S3method(residuals,brma)
S3method(rstandard,brma)
S3method(rstudent,brma)
S3method(summary,brma)
S3method(summary,brma_samples)
S3method(summary,marginal_means.brma)
S3method(summary,zplot_brma)
S3method(summary_heterogeneity,brma)
S3method(summary_models,RoBMA)
S3method(summary_models,default)
S3method(true_effects,brma)
S3method(update,brma)
S3method(vif,brma)
S3method(waic,brma)
S3method(zplot,brma)
S3method(zplot,zplot_brma)
export(BMA)
export(BMA.glmm)
export(BMA.norm)
export(RoBMA)
export(RoBMA.get_option)
export(RoBMA.options)
export(add_loo)
export(add_marglik)
export(add_waic)
export(as_draws)
export(as_draws_array)
export(as_draws_df)
export(as_draws_list)
export(as_draws_matrix)
export(as_draws_rvars)
export(as_zplot)
export(bPEESE)
export(bPET)
export(bayes_factor)
export(bf)
export(blup)
export(bridge_sampler)
export(brma)
export(brma.glmm)
export(brma.norm)
export(bselmodel)
export(check_loo)
export(contr.independent)
export(contr.meandif)
export(contr.orthonormal)
export(covratio)
export(dffits)
export(estimate_unit_information_sd)
export(funnel)
export(galbraith)
export(interpret)
export(logml)
export(loo)
export(loo_compare)
export(loo_weights)
export(marginal_means)
export(plot_diagnostic)
export(plot_diagnostic_autocorrelation)
export(plot_diagnostic_density)
export(plot_diagnostic_trace)
export(plot_pet_peese)
export(plot_prior)
export(plot_weightfunction)
export(pooled_effect)
export(pooled_heterogeneity)
export(post_prob)
export(print_prior)
export(prior)
export(prior_PEESE)
export(prior_PET)
export(prior_factor)
export(prior_informed)
export(prior_none)
export(prior_weightfunction)
export(radial)
export(ranef)
export(regplot)
export(set_autofit_control)
export(set_convergence_checks)
export(summary_heterogeneity)
export(summary_models)
export(true_effects)
export(vif)
export(waic)
export(wf_cumulative)
export(wf_fixed)
export(wf_independent)
export(zplot)
importFrom(BayesTools,is.prior)
importFrom(BayesTools,is.prior.PEESE)
importFrom(BayesTools,is.prior.PET)
importFrom(BayesTools,is.prior.factor)
importFrom(BayesTools,is.prior.independent)
importFrom(BayesTools,is.prior.mixture)
importFrom(BayesTools,is.prior.none)
importFrom(BayesTools,is.prior.point)
importFrom(BayesTools,is.prior.simple)
importFrom(BayesTools,is.prior.spike_and_slab)
importFrom(BayesTools,is.prior.weightfunction)
importFrom(Rdpack,reprompt)
importFrom(bridgesampling,bayes_factor)
importFrom(bridgesampling,bf)
importFrom(bridgesampling,bridge_sampler)
importFrom(bridgesampling,logml)
importFrom(bridgesampling,post_prob)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(rlang,.data)
importFrom(stats,coef)
importFrom(stats,cooks.distance)
importFrom(stats,dfbetas)
importFrom(stats,fitted)
importFrom(stats,hatvalues)
importFrom(stats,influence)
importFrom(stats,logLik)
importFrom(stats,model.matrix)
importFrom(stats,nobs)
importFrom(stats,plogis)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,qqnorm)
importFrom(stats,residuals)
importFrom(stats,rstandard)
importFrom(stats,rstudent)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,capture.output)
importFrom(utils,getS3method)
