## ----vignette-nowrap-options, include = FALSE, eval = [encoded data, 9944 characters]
options(width = 10000)

## ----setup, include = [encoded data, 9976 characters]
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)

article_url <- function(page) {
  paste0(page, ".html")
}

reference_url <- function(page) {
  paste0("../reference/", page, ".html")
}

