```{r vignette-nowrap-options, include = FALSE, eval = TRUE}
options(width = 10000)
```

<style type="text/css">
pre,
pre code,
pre:not([class]),
pre.sourceCode,
pre.sourceCode code,
pre > code.sourceCode {
  white-space: pre !important;
  word-break: normal !important;
  overflow-wrap: normal !important;
  word-wrap: normal !important;
}

pre,
div.sourceCode {
  overflow-x: auto !important;
}
</style>
