% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Weingarten2018}
\alias{Weingarten2018}
\title{582 effect sizes examining the ease-of-retrieval effect
from a meta-analysis by \insertCite{weingarten2018does;textual}{RoBMA}}
\format{
A data.frame with 12 columns and 582 observations:
\describe{
\item{\code{r_xy}}{Correlation between manipulation and outcome.}
\item{\code{N}}{Sample size.}
\item{\code{paper_id}}{Paper identifier.}
\item{\code{published}}{Publication-status indicator.}
\item{\code{USA}}{Whether the study was conducted in the USA.}
\item{\code{number_of_few}}{Number of examples in the few-examples condition.}
\item{\code{number_of_many}}{Number of examples in the many-examples condition.}
\item{\code{episodic_memory}}{Whether the trial targeted episodic memory.}
\item{\code{standard_paradigm}}{Whether the standard paradigm was used.}
\item{\code{proximal_dataset}}{Whether the data set was proximal.}
\item{\code{r_xm}}{Correlation between manipulation and mediator.}
\item{\code{r_my}}{Correlation between mediator and outcome.}
}
}
\usage{
Weingarten2018
}
\description{
The data set contains correlation coefficients between the manipulation
and outcome variable (r_xy), sample sizes (N), and various study characteristics
including publication status, country (USA vs. other), number of few and many
examples requested, whether the trial targeted episodic memory, paradigm type (standard vs. other),
dataset type (proximal vs. distal), and mediation variables (r_xm, r_my).
The meta-analysis examined whether subjective ease mediates the ease-of-retrieval effect,
where participants list either few or many examples and then make judgments
\insertCite{weingarten2018does}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
