% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{loo_compare.loo}
\alias{loo_compare.loo}
\title{Compare loo Objects Using LOO}
\usage{
\method{loo_compare}{loo}(x, ..., unit = "estimate")
}
\arguments{
\item{x}{a RoBMA-targeted \code{loo} or \code{waic} object (the first model
to compare).}

\item{...}{additional RoBMA-targeted \code{loo}/\code{waic} or \code{brma}
objects to compare.}

\item{unit}{output/deletion unit used when extracting LOO from brma objects.}
}
\value{
A matrix of class \code{"compare.loo"} as returned by
\code{\link[loo]{loo_compare}}.
}
\description{
Method for comparing RoBMA-targeted \code{loo} or \code{waic}
objects directly.
}
\seealso{
\code{\link{loo.brma}}, \code{\link[loo]{loo_compare}}
}
