% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brma_samples.R
\name{print.summary.brma_samples}
\alias{print.summary.brma_samples}
\title{Print summary.brma_samples Object}
\usage{
\method{print}{summary.brma_samples}(x, probs = NULL, ...)
}
\arguments{
\item{x}{a \code{summary.brma_samples} object}

\item{probs}{quantiles for credible intervals. If \code{NULL}, uses
the default stored in the object (typically \code{c(.025, .975)})}

\item{...}{additional arguments passed to
\code{BayesTools::ensemble_estimates_table}}
}
\value{
Returns the summary table invisibly.
}
\description{
Prints a summary table of posterior samples using
\code{BayesTools::ensemble_estimates_table}. Returns the summary table
invisibly.
}
