% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{logLik.brma}
\alias{logLik.brma}
\title{Extract Log-Likelihood Matrix from brma Object}
\usage{
\method{logLik}{brma}(object, unit = "estimate", ...)
}
\arguments{
\item{object}{a brma model object.}

\item{unit}{output unit. See \code{\link{add_loo}}.}

\item{...}{currently unused.}
}
\value{
An S x K or S x G matrix of log-likelihood values.
}
\description{
Extract the pointwise log-likelihood matrix from a brma model
object. This is an S x K or S x G matrix where S is the number of posterior
samples, K is the number of estimates, and G is the number of clusters.
This method implements the S3
\'logLik\' generic for \code{brma} objects and returns the matrix of
pointwise log-likelihoods (one column per observation, one row per sample).
}
\details{
The log-likelihood is computed for each observation at each posterior sample.
For binomial and Poisson models, each observation consists of a pair of
counts (ai/ci or x1i/x2i) that together define a single effect size estimate.
}
\seealso{
\code{\link{loo.brma}}
}
