% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Anderson2010}
\alias{Anderson2010}
\title{23 experimental studies from
\insertCite{anderson2010violent;textual}{RoBMA} that meet the best practice criteria}
\format{
A data.frame with 3 columns and 23 observations:
\describe{
\item{\code{r}}{Correlation coefficient.}
\item{\code{n}}{Sample size.}
\item{\code{name}}{Study label.}
}
}
\source{
\url{https://github.com/Joe-Hilgard/Anderson-meta}
}
\usage{
Anderson2010
}
\description{
The data set contains correlation coefficients, sample
sizes, and labels for 23 experimental studies focusing on the effect of
violent video games on aggressive behavior.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
