% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_PEESE}
\alias{prior_PEESE}
\title{PEESE Prior}
\usage{
prior_PEESE(
  distribution,
  parameters,
  truncation = list(lower = 0, upper = Inf),
  prior_weights = 1
)
}
\arguments{
\item{distribution}{character. Prior distribution name.}

\item{parameters}{list. Distribution parameters.}

\item{truncation}{list with \code{lower} and \code{upper} truncation bounds.}

\item{prior_weights}{numeric prior model weight.}
}
\value{
An object inheriting from \code{prior} with the \code{prior.PEESE}
marker class.
}
\description{
Create PEESE publication-bias regression priors.
}
\details{
This forwards to \code{BayesTools::prior_PEESE()} and uses the same
distribution and parameter conventions as \code{prior()}.
}
\seealso{
\code{\link{publication_bias_prior_specification}}
}
