% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Havrankova2025}
\alias{Havrankova2025}
\title{1159 effect sizes from a meta-analysis of beauty and professional success
by \insertCite{havrankova2025beauty;textual}{RoBMA}}
\format{
A data.frame with 5 columns and 1159 observations:
\describe{
\item{\code{y}}{Effect size: percent increase in earnings.}
\item{\code{se}}{Standard error of y.}
\item{\code{facing_customer}}{Type of customer contact.}
\item{\code{study_id}}{Study identifier.}
\item{\code{N}}{Sample size.}
}
}
\usage{
Havrankova2025
}
\description{
The data set contains effect sizes (percent increase in earnings), standard errors,
study identifiers, sample sizes, and the type of customer contact (no, some, or direct).
The meta-analysis examined the relationship between perceived beauty and professional success
\insertCite{havrankova2025beauty}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
