% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vif.R
\name{vif}
\alias{vif}
\title{Variance Inflation Factors}
\usage{
vif(object, ...)
}
\arguments{
\item{object}{a fitted model object}

\item{...}{additional arguments passed to methods}
}
\value{
Method-specific return value containing VIF diagnostics.
}
\description{
Computes variance inflation factors (VIF) for a fitted model
with moderators, to assess multicollinearity among predictors.
}
\references{
\insertCite{fox1992generalized}{RoBMA}
}
\seealso{
\code{\link[=regplot]{regplot()}}, \code{\link[=summary.brma]{summary.brma()}}
}
