% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{coef.brma}
\alias{coef.brma}
\title{Extract Model Coefficients for brma Objects}
\usage{
\method{coef}{brma}(object, ...)
}
\arguments{
\item{object}{a fitted brma object}

\item{...}{additional arguments (currently ignored)}
}
\value{
A named numeric vector of posterior mean coefficients.
}
\description{
Extract model coefficients (posterior means) from a
fitted brma object.
}
\seealso{
\code{\link[=summary.brma]{summary.brma()}}
}
