% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_heterogeneity.R
\name{print.summary_heterogeneity.brma}
\alias{print.summary_heterogeneity.brma}
\title{Print Summary of Heterogeneity}
\usage{
\method{print}{summary_heterogeneity.brma}(x, ...)
}
\arguments{
\item{x}{a \code{summary_heterogeneity.brma} object}

\item{...}{additional arguments (currently ignored)}
}
\value{
Returns \code{x} invisibly.
}
\description{
Prints the heterogeneity summary table.
}
