% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vif.R
\name{print.vif.brma}
\alias{print.vif.brma}
\title{Print VIF Results}
\usage{
\method{print}{vif.brma}(x, digits = 3, ...)
}
\arguments{
\item{x}{a \code{vif.brma} object}

\item{digits}{number of decimal places. Defaults to 3.}

\item{...}{additional arguments (currently ignored)}
}
\value{
Returns \code{x} invisibly.
}
\description{
Prints variance inflation factors and optional posterior
correlation matrix.
}
