% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_none}
\alias{prior_none}
\title{Empty Prior}
\usage{
prior_none(prior_weights = 1)
}
\arguments{
\item{prior_weights}{numeric prior model weight.}
}
\value{
An object inheriting from \code{prior} and \code{prior.none}.
}
\description{
Create an empty prior used to omit a model component.
}
