% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bridgesampling.R
\name{logml.brma}
\alias{logml.brma}
\title{Log Marginal Likelihood for brma Objects}
\usage{
\method{logml}{brma}(x, ...)
}
\arguments{
\item{x}{a brma model object.}

\item{...}{additional arguments (currently not used).}
}
\value{
A scalar numeric value representing the log marginal likelihood.
}
\description{
Extract the log marginal likelihood from a brma model.
The marginal likelihood must first be computed using \code{\link{add_marglik}}.
}
\details{
This function extracts the log marginal likelihood from the bridge sampling
object that was previously computed and stored using \code{\link{add_marglik}}.
Product-space model-averaging objects (\code{BMA.norm}, \code{BMA.glmm},
and \code{RoBMA}) do not expose bridge-sampling marginal likelihoods.
}
\examples{
\dontrun{
if (requireNamespace("metadat", quietly = TRUE)) {
  data(dat.lehmann2018, package = "metadat")
  fit <- brma(yi = yi, vi = vi, data = dat.lehmann2018, measure = "SMD")

  fit <- add_marglik(fit)

  logml(fit)
}
}

}
\seealso{
\code{\link{add_marglik}}, \code{\link{bridge_sampler.brma}},
\code{\link{bf.brma}}, \code{\link{post_prob.brma}}
}
