% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marginal_means.R
\name{marginal_means}
\alias{marginal_means}
\title{Estimated Marginal Means}
\usage{
marginal_means(object, ...)
}
\arguments{
\item{object}{a fitted model object.}

\item{...}{additional arguments passed to methods.}
}
\value{
A method-specific estimated marginal means object.
}
\description{
S3 generic for estimated marginal means. The \code{brma}
method works with fitted moderator models and stores the BayesTools
marginal-inference object for \code{summary()} and \code{plot()}.
}
\seealso{
\code{\link[=summary]{summary()}}, \code{\link[=plot]{plot()}}, \code{\link[=summary.brma]{summary.brma()}}, \code{\link[=regplot]{regplot()}}
}
