% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marginal_means.R
\name{print.summary.marginal_means.brma}
\alias{print.summary.marginal_means.brma}
\title{Print Summary of Estimated Marginal Means}
\usage{
\method{print}{summary.marginal_means.brma}(x, ...)
}
\arguments{
\item{x}{a \code{summary.marginal_means.brma} object.}

\item{...}{additional arguments (currently ignored).}
}
\value{
Returns \code{x} invisibly.
}
\description{
Prints a summary table of estimated marginal means.
}
