% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{pooled_heterogeneity}
\alias{pooled_heterogeneity}
\title{Pooled Heterogeneity}
\usage{
pooled_heterogeneity(object, ...)
}
\arguments{
\item{object}{a fitted model object}

\item{...}{additional arguments passed to methods}
}
\value{
Method-specific return value, typically a summary table or
posterior samples of the pooled heterogeneity.
}
\description{
Computes the pooled (aggregated) heterogeneity estimate (tau)
from a fitted model.
}
\seealso{
\code{\link[=predict.brma]{predict.brma()}}
}
