% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input-data.R
\name{print.RoBMA_data}
\alias{print.RoBMA_data}
\title{Print method for RoBMA_data objects}
\usage{
\method{print}{RoBMA_data}(x, n = 6, ...)
}
\arguments{
\item{x}{an RoBMA_data object}

\item{n}{maximum number of rows to display for each component. Defaults to 6.}

\item{...}{additional arguments (ignored)}
}
\value{
Invisibly returns the input object.
}
\description{
Prints a concise summary of an RoBMA_data object.
}
