% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Poulsen2006}
\alias{Poulsen2006}
\title{5 studies with a tactile outcome assessment from
\insertCite{poulsen2006potassium;textual}{RoBMA} of the effect of potassium-containing toothpaste
on dentine hypersensitivity}
\format{
A data.frame with 3 columns and 5 observations:
\describe{
\item{\code{d}}{Cohen's d effect size.}
\item{\code{se}}{Standard error of d.}
\item{\code{study}}{Study label.}
}
}
\usage{
Poulsen2006
}
\description{
The data set contains Cohen's d effect sizes, standard errors,
and labels for 5 studies assessing the tactile outcome from a meta-analysis of
the effect of potassium-containing toothpaste on dentine hypersensitivity
\insertCite{poulsen2006potassium}{RoBMA} which was used as an example in
\insertCite{bartos2021bayesian;textual}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
