% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{nobs.brma}
\alias{nobs.brma}
\title{Number of Observations for brma Objects}
\usage{
\method{nobs}{brma}(object, ...)
}
\arguments{
\item{object}{a fitted brma object}

\item{...}{additional arguments (currently ignored)}
}
\value{
An integer giving the number of observations.
}
\description{
Extract the number of observations (studies/estimates) from
a fitted brma object.
}
\seealso{
\code{\link[=brma]{brma()}}, \code{\link[=brma.glmm]{brma.glmm()}}
}
