% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brma_samples.R
\name{as.matrix.brma_samples}
\alias{as.matrix.brma_samples}
\title{Convert brma_samples to Matrix}
\usage{
\method{as.matrix}{brma_samples}(x, ...)
}
\arguments{
\item{x}{a \code{brma_samples} object}

\item{...}{additional arguments (ignored)}
}
\value{
A plain matrix of posterior samples.
}
\description{
Converts a \code{brma_samples} object to a plain matrix,
removing all brma_samples-specific attributes.
}
