% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Johnides2025}
\alias{Johnides2025}
\title{412 effect sizes from a meta-analysis of secondary benefits of family-based treatments
by \insertCite{johnides2025secondary;textual}{RoBMA}}
\format{
A data.frame with 3 columns and 412 observations:
\describe{
\item{\code{study}}{Study label.}
\item{\code{d}}{Cohen's d effect size.}
\item{\code{se}}{Standard error of d.}
}
}
\usage{
Johnides2025
}
\description{
The data set contains Cohen's d effect sizes, standard errors, and study labels
from a meta-analysis investigating the extent to which family-based treatments for children
with mental health, physical health, and developmental disorders provide secondary benefits
to the children's siblings and caregivers \insertCite{johnides2025secondary}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
