% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RoBMA-package.R
\docType{package}
\name{RoBMA-package}
\alias{RoBMA-package}
\alias{RoBMA_package}
\alias{RoBMA.package}
\title{RoBMA: Robust Bayesian Meta-Analysis}
\description{
RoBMA provides Bayesian meta-analysis, meta-regression, multilevel
meta-analysis, model averaging, and publication-bias adjustment. The main
user-facing fitters are \code{\link{RoBMA}}, \code{\link{BMA}},
\code{\link{brma}}, \code{\link{brma.glmm}}, \code{\link{bselmodel}},
\code{\link{bPET}}, and \code{\link{bPEESE}}.
}
\section{User guide}{

See \insertCite{bartos2021no;textual}{RoBMA},
\insertCite{maier2020robust;textual}{RoBMA},
\insertCite{bartos2020adjusting;textual}{RoBMA}, and
\insertCite{bartos2025robust;textual}{RoBMA} for the RoBMA methodology.
Use \code{vignette(package = "RoBMA")} to list installed vignettes.
}

\references{
\insertAllCited{}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://fbartos.github.io/RoBMA/}
  \item Report bugs at \url{https://github.com/FBartos/RoBMA/issues}
}

}
\author{
Frantisek Bartos \email{f.bartos96@gmail.com}
}
\keyword{package}
