% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{true_effects.brma}
\alias{true_effects.brma}
\title{True Effects for brma Objects}
\usage{
\method{true_effects}{brma}(
  object,
  bias_adjusted = FALSE,
  output_measure = NULL,
  transform = NULL,
  probs = c(0.025, 0.975),
  ...
)
}
\arguments{
\item{object}{a fitted brma object}

\item{bias_adjusted}{whether to adjust for publication bias. Defaults to
\code{FALSE}, which returns estimates including publication bias effects
(i.e., what we expect the true effects to be given the biased observations).
Set to \code{TRUE} to obtain bias-corrected estimates.}

\item{output_measure}{effect-size measure for location/effect predictions.
Defaults to the fitted measure. Supported conversions are among \code{"SMD"},
\code{"COR"}, \code{"ZCOR"}, and \code{"OR"}; \code{"RR"}, \code{"HR"},
\code{"IRR"}, \code{"RD"}, and \code{"GEN"} can only be returned on their
fitted measure. Use \code{transform = "EXP"} for ratio-scale output from
log-scale measures.}

\item{transform}{optional display transformation. Currently \code{"EXP"}
exponentiates log-scale measures \code{"OR"}, \code{"RR"}, \code{"HR"},
and \code{"IRR"}.}

\item{probs}{quantiles of the posterior distribution to be displayed.
Defaults to \code{c(.025, .975)} for 95\% credible intervals.}

\item{...}{additional arguments passed to \code{\link{predict.brma}}; wrapper
arguments such as \code{newdata}, \code{type}, \code{quiet},
\code{output_measure}, and \code{transform} are fixed by this method.}
}
\value{
A \code{brma_samples} object containing posterior draws of BLUP or
empirical-Bayes true-effect summaries with one column per estimate. For
existing normal data, these are conditional BLUP means, not simulated
latent-effect draws. When printed, displays a summary table. Use
\code{summary()} to obtain the summary table directly. The samples can be
converted to \pkg{posterior} draws formats using \code{as_draws()}.
}
\description{
Computes the estimated true effects (theta) for a
fitted brma object. This is an alias for \code{\link{blup.brma}}.
}
\details{
This function is identical to \code{\link{blup.brma}}. See that function
for full details on how true effects are computed.
}
\examples{
\dontrun{
if (requireNamespace("metadat", quietly = TRUE)) {
  data(dat.lehmann2018, package = "metadat")
  fit <- brma(
    yi      = yi,
    vi      = vi,
    data    = dat.lehmann2018,
    measure = "SMD",
    seed    = 1,
    silent  = TRUE
  )

  true_effects(fit)
}
}

}
\seealso{
\code{\link[=blup.brma]{blup.brma()}}, \code{\link[=predict.brma]{predict.brma()}}, \code{\link[=pooled_effect]{pooled_effect()}},
\code{\link[=pooled_heterogeneity]{pooled_heterogeneity()}}
}
