% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Hoppen2025}
\alias{Hoppen2025}
\title{37 studies from a meta-analysis of social comparison as a behavior change technique
by \insertCite{hoppen2025meta;textual}{RoBMA}}
\format{
A data.frame with 9 columns and 37 observations:
\describe{
\item{\code{d}}{Cohen's d effect size.}
\item{\code{v}}{Sampling variance of d.}
\item{\code{outcome}}{Outcome type.}
\item{\code{feedback_level}}{Feedback level.}
\item{\code{social_comparison_type}}{Social-comparison type.}
\item{\code{sessions}}{Number of sessions.}
\item{\code{sample_type}}{Sample type.}
\item{\code{sample_size}}{Sample size.}
\item{\code{country}}{Country.}
}
}
\usage{
Hoppen2025
}
\description{
The data set contains Cohen's d effect sizes, variances, and study
characteristics including outcome type, feedback level, social comparison type,
number of sessions, sample type, sample size, and country. The meta-analysis
examined social comparison as a behavior change technique across the behavioral
sciences \insertCite{hoppen2025meta}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
