% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Lui2015}
\alias{Lui2015}
\title{18 studies of a relationship between acculturation mismatch and
intergenerational cultural conflict collected by
\insertCite{lui2015intergenerational;textual}{RoBMA}}
\format{
A data.frame with 3 columns and 18 observations:
\describe{
\item{\code{r}}{Correlation coefficient.}
\item{\code{n}}{Sample size.}
\item{\code{study}}{Study label.}
}
}
\usage{
Lui2015
}
\description{
The data set contains correlation coefficients r,
sample sizes n, and labels for each study assessing the
relationship between acculturation mismatch (that is the result of the contrast
between the collectivist cultures of Asian and Latin immigrant groups
and the individualist culture in the United States) and intergenerational cultural
conflict \insertCite{lui2015intergenerational}{RoBMA} which was used as an
example in \insertCite{bartos2020adjusting;textual}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
