% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.brma}
\alias{summary.brma}
\alias{print.summary.brma}
\alias{print.brma}
\title{Summarize brma Object}
\usage{
\method{summary}{brma}(
  object,
  probs = c(0.025, 0.5, 0.975),
  include_mcmc_diagnostics = TRUE,
  standardized_coefficients = FALSE,
  conditional = FALSE,
  logBF = FALSE,
  BF01 = FALSE,
  ...
)

\method{print}{summary.brma}(x, ...)

\method{print}{brma}(x, ...)
}
\arguments{
\item{object}{a fitted brma object}

\item{probs}{quantiles of the posterior samples to be displayed.
Defaults to \code{c(.025, .50, .975)}}

\item{include_mcmc_diagnostics}{whether to include MCMC diagnostics in the output.
Defaults to \code{TRUE}.}

\item{standardized_coefficients}{whether to show standardized meta-regression coefficients.
Defaults to \code{FALSE}. When set to \code{TRUE}, standardized meta-regression
coefficients are returned for the intercept and continuous predictors. These coefficients
correspond to the standardized scale on which prior distributions are specified by default
(i.e., \code{standardize_continuous_predictors = TRUE}).}

\item{conditional}{whether to include conditional estimates for RoBMA
product-space objects. Defaults to \code{FALSE}.}

\item{logBF}{whether to show inclusion Bayes factors on the log scale.
Defaults to \code{FALSE}.}

\item{BF01}{whether to show inverse inclusion Bayes factors. Defaults to
\code{FALSE}.}

\item{...}{additional arguments}

\item{x}{a \code{summary.brma} or fitted \code{brma} object.}
}
\value{
A list of class \code{summary.brma} with model name, optional RoBMA
inclusion tables, common estimates, moderator estimates, scale estimates,
publication-bias estimates, and optional conditional estimates. The printed
form displays the non-empty tables.
}
\description{
\code{summary.brma} creates summary tables for a
brma object. For RoBMA objects, inclusion summaries are printed before
parameter estimates.
}
\examples{
\dontrun{
if (requireNamespace("metadat", quietly = TRUE)) {
  data(dat.lehmann2018, package = "metadat")

  fit <- bPET(
    yi      = yi,
    vi      = vi,
    data    = dat.lehmann2018,
    measure = "SMD",
    seed    = 1,
    silent  = TRUE
  )

  summary(fit)
}
}


}
\seealso{
\code{\link[=brma]{brma()}}, \code{\link[=brma.glmm]{brma.glmm()}}
}
