% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brma_samples.as_draws.R
\name{as_draws.brma_samples}
\alias{as_draws.brma_samples}
\alias{as_draws_array.brma_samples}
\alias{as_draws_df.brma_samples}
\alias{as_draws_list.brma_samples}
\alias{as_draws_matrix.brma_samples}
\alias{as_draws_rvars.brma_samples}
\title{Convert brma_samples to posterior Draws Formats}
\usage{
\method{as_draws}{brma_samples}(x, ...)

\method{as_draws_array}{brma_samples}(x, ...)

\method{as_draws_df}{brma_samples}(x, ...)

\method{as_draws_list}{brma_samples}(x, ...)

\method{as_draws_matrix}{brma_samples}(x, ...)

\method{as_draws_rvars}{brma_samples}(x, ...)
}
\arguments{
\item{x}{a \code{brma_samples} object}

\item{...}{additional arguments passed to the corresponding
\pkg{posterior} function}
}
\value{
An object of the corresponding \pkg{posterior} draws class.
}
\description{
Provides an interface to the \pkg{posterior} package
for \code{brma_samples} objects. These functions convert the posterior
samples to various draws formats supported by the \pkg{posterior} package.
}
\details{
The conversion reconstructs the MCMC chain structure from the stored
\code{nchains} and \code{niter} attributes. The samples are assumed
to be ordered with chains concatenated (i.e., all iterations from chain 1,
then all from chain 2, etc.). Conditional RoBMA samples are intentionally
stored as one flattened chain because conditioning subsets posterior rows
across chains.
}
\seealso{
\code{\link[posterior]{draws}}, \code{\link{as_draws.brma}}
}
