% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_heterogeneity.R
\name{summary_heterogeneity}
\alias{summary_heterogeneity}
\title{Summary of Heterogeneity}
\usage{
summary_heterogeneity(object, ...)
}
\arguments{
\item{object}{a fitted model object}

\item{...}{additional arguments passed to methods}
}
\value{
Method-specific return value containing heterogeneity estimates.
}
\description{
Computes the absolute heterogeneity (tau, tau^2) and
relative measures of heterogeneity (I^2, H^2) for a fitted model.
}
\seealso{
\code{\link[=pooled_heterogeneity]{pooled_heterogeneity()}}, \code{\link[=summary.brma]{summary.brma()}}
}
