% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{ManyLabs16}
\alias{ManyLabs16}
\title{55 effect sizes from Many Labs 2 replication studies of
\insertCite{tversky1981framing;textual}{RoBMA} framing effects}
\format{
A data.frame with 2 columns and 55 observations:
\describe{
\item{\code{y}}{Standardized mean difference.}
\item{\code{se}}{Standard error of y.}
}
}
\usage{
ManyLabs16
}
\description{
The data set contains standardized mean differences (y) and standard errors (se)
from 55 replication studies of the framing effect originally described by
\insertCite{tversky1981framing;textual}{RoBMA}. These studies were part of the
Many Labs 2 project examining variation in replicability across samples and settings
\insertCite{klein2018many;textual}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
