% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{Bem2011}
\alias{Bem2011}
\title{9 experimental studies from
\insertCite{bem2011feeling;textual}{RoBMA} as described in
\insertCite{bem2011must;textual}{RoBMA}}
\format{
A data.frame with 3 columns and 9 observations:
\describe{
\item{\code{d}}{Cohen's d effect size.}
\item{\code{se}}{Standard error of d.}
\item{\code{study}}{Study label.}
}
}
\usage{
Bem2011
}
\description{
The data set contains Cohen's d effect sizes, standard errors,
and labels for 9 experimental studies of precognition from the infamous
\insertCite{bem2011feeling;textual}{RoBMA} as analyzed in his later meta-analysis
\insertCite{bem2011must}{RoBMA}.
}
\references{
\insertAllCited{}
}
\keyword{datasets}
