% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior}
\alias{prior}
\title{Prior Distribution}
\usage{
prior(
  distribution,
  parameters,
  truncation = list(lower = -Inf, upper = Inf),
  prior_weights = 1
)
}
\arguments{
\item{distribution}{character. Prior distribution name.}

\item{parameters}{list. Distribution parameters.}

\item{truncation}{list with \code{lower} and \code{upper} truncation bounds.}

\item{prior_weights}{numeric prior model weight.}
}
\value{
An object inheriting from \code{prior}.
}
\description{
Create prior distribution objects used by RoBMA and brma
fitting functions.
}
\details{
This is RoBMA's re-export of \code{BayesTools::prior()}; supported
distribution names and parameter lists follow BayesTools.
}
\examples{
prior("normal", list(mean = 0, sd = 0.5))
prior(
  "normal",
  list(mean = 0, sd = 0.5),
  truncation = list(lower = 0, upper = Inf)
)

}
