% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zplot.R
\name{print.summary.zplot_brma}
\alias{print.summary.zplot_brma}
\title{Print Zplot Summary}
\usage{
\method{print}{summary.zplot_brma}(x, ...)
}
\arguments{
\item{x}{a summary.zplot_brma object.}

\item{...}{additional arguments (currently unused).}
}
\value{
Invisibly returns the summary object.
}
\description{
Print Zplot Summary
}
