% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrappers.R
\name{blup}
\alias{blup}
\title{Best Linear Unbiased Predictions (BLUPs)}
\usage{
blup(object, ...)
}
\arguments{
\item{object}{a fitted model object}

\item{...}{additional arguments passed to methods}
}
\value{
Method-specific return value, typically a summary table or
posterior samples of BLUP or empirical-Bayes true-effect summaries.
}
\description{
Computes the estimated true effects (theta) from a
fitted model. These correspond to Best Linear Unbiased Predictions (BLUPs)
or empirical Bayes estimates.
}
\seealso{
\code{\link[=predict.brma]{predict.brma()}}
}
