% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{add_waic.brma}
\alias{add_waic.brma}
\alias{add_waic}
\title{Add WAIC to brma Objects}
\usage{
\method{add_waic}{brma}(object, unit = "estimate", ...)
}
\arguments{
\item{object}{a brma model object.}

\item{unit}{output/deletion unit. See \code{\link{add_loo}}; the same
accepted values and multilevel constraint apply.}

\item{...}{additional arguments passed to \code{\link[loo]{waic}}.}
}
\value{
The brma object with the WAIC result stored in
\code{object[["waic"]][[unit]]}.
}
\description{
Compute the Widely Applicable Information Criterion (WAIC)
for brma model objects and store the result in the object.
}
\details{
WAIC is an alternative to LOO-CV for estimating out-of-sample predictive
accuracy. Like LOO, it evaluates expected predictive performance for new
observations.

In most cases, LOO-PSIS (via \code{\link{add_loo}}) is preferred over WAIC
because it provides better estimates and includes diagnostics (Pareto k
values) that indicate when the approximation may be unreliable.
}
\seealso{
\code{\link{waic.brma}}, \code{\link{add_loo}}, \code{\link[loo]{waic}}
}
