% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marginal_means.R
\name{print.marginal_means.brma}
\alias{print.marginal_means.brma}
\title{Print Estimated Marginal Means}
\usage{
\method{print}{marginal_means.brma}(x, ...)
}
\arguments{
\item{x}{a \code{marginal_means.brma} object.}

\item{...}{additional arguments passed to \code{summary()}.}
}
\value{
Returns \code{x} invisibly.
}
\description{
Prints the estimated marginal means summary.
}
