% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bridgesampling.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{bridge_sampler}
\alias{logml}
\alias{post_prob}
\alias{bf}
\alias{bayes_factor}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{bridgesampling}{\code{\link[bridgesampling:bf]{bayes_factor}}, \code{\link[bridgesampling]{bf}}, \code{\link[bridgesampling]{bridge_sampler}}, \code{\link[bridgesampling]{logml}}, \code{\link[bridgesampling]{post_prob}}}
}}

