% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{check_loo.brma}
\alias{check_loo.brma}
\alias{check_loo}
\title{Check LOO Diagnostics for brma Objects}
\usage{
\method{check_loo}{brma}(object, unit = "estimate", ...)
}
\arguments{
\item{object}{a brma model object.}

\item{unit}{output/deletion unit. See \code{\link{add_loo}}.}

\item{...}{currently unused.}
}
\value{
NULL (throws warning if diagnostics are unreliable).
}
\description{
Check Pareto k diagnostics for a brma model object and
warn if any values are high.
}
\details{
LOO must first be computed with
\code{object <- add_loo(object, unit = unit)}. The method warns when any
Pareto \eqn{k} diagnostic is greater than 0.7.
}
