% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loo.R
\name{loo_weights.brma}
\alias{loo_weights.brma}
\alias{loo_weights}
\title{Extract Normalized PSIS Weights from brma Object}
\usage{
\method{loo_weights}{brma}(object, unit = "estimate", ...)
}
\arguments{
\item{object}{a brma model object.}

\item{unit}{output/deletion unit. See \code{\link{add_loo}}.}

\item{...}{currently unused.}
}
\value{
An \code{S x K} matrix for estimate-unit LOO, or \code{S x G}
matrix for cluster-unit LOO, with posterior samples in rows and LOO targets
in columns. Columns are normalized to sum to one.
}
\description{
Extract the normalized Pareto smoothed importance sampling
(PSIS) weights from a brma model object.
}
\details{
LOO must first be computed with
\code{object <- add_loo(object, unit = unit)}. This method extracts the
stored PSIS object and does not compute LOO.
}
